@@ -1,4 +1,5 @@
 from collections import defaultdict
+from typing import Dict, List, Set  # noqa
 
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
@@ -10,7 +11,8 @@ from django.utils.translation import ugettext as _
 from django.utils.translation import ugettext_lazy
 
 from openslides.core.config import config
-from openslides.core.models import Countdown
+from openslides.core.models import Countdown, Projector
+from openslides.utils.autoupdate import inform_changed_data
 from openslides.utils.exceptions import OpenSlidesError
 from openslides.utils.models import RESTModelMixin
 from openslides.utils.utils import to_roman
@@ -79,7 +81,7 @@ class ItemManager(models.Manager):
         HIDDEN_ITEM and all of their children.
         """
         queryset = self.order_by('weight')
-        item_children = defaultdict(list)
+        item_children = defaultdict(list)  # type: Dict[int, List[Item]]
         root_items = []
         for item in queryset:
             if only_agenda_items and item.type == item.HIDDEN_ITEM:
@@ -135,7 +137,7 @@ class ItemManager(models.Manager):
                 yield (element['id'], parent, weight)
                 yield from walk_items(element.get('children', []), element['id'])
 
-        touched_items = set()
+        touched_items = set()  # type: Set[int]
         db_items = dict((item.pk, item) for item in Item.objects.all())
         for item_id, parent_id, weight in walk_items(tree):
             # Check that the item is only once in the tree to prevent invalid trees
@@ -278,12 +280,24 @@ class Item(RESTModelMixin, models.Model):
         permissions = (
             ('can_see', 'Can see agenda'),
             ('can_manage', 'Can manage agenda'),
+            ('can_manage_list_of_speakers', 'Can manage list of speakers'),
             ('can_see_hidden_items', 'Can see hidden items and time scheduling of agenda'))
         unique_together = ('content_type', 'object_id')
 
     def __str__(self):
         return self.title
 
+    def delete(self, skip_autoupdate=False, *args, **kwargs):
+        """
+        Customized method to delete an agenda item. Ensures that a respective
+        list of speakers projector element is disabled.
+        """
+        Projector.remove_any(
+            skip_autoupdate=skip_autoupdate,
+            name='agenda/list-of-speakers',
+            id=self.pk)
+        return super().delete(skip_autoupdate=skip_autoupdate, *args, **kwargs)  # type: ignore
+
     @property
     def title(self):
         """
@@ -387,6 +401,11 @@ class Speaker(RESTModelMixin, models.Model):
     The sort order of the list of speakers. None, if he has already spoken.
     """
 
+    marked = models.BooleanField(default=False)
+    """
+    Marks a speaker.
+    """
+
     class Meta:
         default_permissions = ()
         permissions = (
@@ -409,31 +428,35 @@ class Speaker(RESTModelMixin, models.Model):
         except Speaker.DoesNotExist:
             pass
         else:
-            current_speaker.end_speech()
+            # Do not send an autoupdate for the countdown and the item. This is done
+            # by saving the item and countdown later.
+            current_speaker.end_speech(skip_autoupdate=True)
         self.weight = None
         self.begin_time = timezone.now()
-        self.save()
+        self.save()  # Here, the item is saved and causes an autoupdate.
         if config['agenda_couple_countdown_and_speakers']:
             countdown, created = Countdown.objects.get_or_create(pk=1, defaults={
                 'default_time': config['projector_default_countdown'],
                 'countdown_time': config['projector_default_countdown']})
             if not created:
-                countdown.control(action='reset')
-            countdown.control(action='start')
+                countdown.control(action='reset', skip_autoupdate=True)
+            countdown.control(action='start', skip_autoupdate=True)
 
-    def end_speech(self):
+            inform_changed_data(countdown)  # Here, the autoupdate for the countdown is triggered.
+
+    def end_speech(self, skip_autoupdate=False):
         """
         The speech is finished. Set the time to now.
         """
         self.end_time = timezone.now()
-        self.save()
+        self.save(skip_autoupdate=skip_autoupdate)
         if config['agenda_couple_countdown_and_speakers']:
             try:
                 countdown = Countdown.objects.get(pk=1)
             except Countdown.DoesNotExist:
                 pass  # Do not create a new countdown on stop action
             else:
-                countdown.control(action='stop')
+                countdown.control(action='reset', skip_autoupdate=skip_autoupdate)
 
     def get_root_rest_element(self):
         """
