@@ -1,35 +1,33 @@
-import base64
 import json
-import os
 import uuid
-from collections import OrderedDict
-from operator import attrgetter
 from textwrap import dedent
+from typing import Any, Dict, List, cast  # noqa
 
 from django.apps import apps
 from django.conf import settings
-from django.contrib.staticfiles import finders
 from django.db.models import F
 from django.http import Http404, HttpResponse
 from django.utils.timezone import now
 from django.utils.translation import ugettext as _
+from mypy_extensions import TypedDict
 
+from .. import __license__ as license
+from .. import __url__ as url
 from .. import __version__ as version
 from ..utils import views as utils_views
 from ..utils.auth import anonymous_is_enabled, has_perm
 from ..utils.autoupdate import inform_changed_data, inform_deleted_data
-from ..utils.collection import Collection, CollectionElement
 from ..utils.plugins import (
     get_plugin_description,
+    get_plugin_license,
+    get_plugin_url,
     get_plugin_verbose_name,
     get_plugin_version,
 )
 from ..utils.rest_api import (
     ModelViewSet,
     Response,
-    SimpleMetadata,
     ValidationError,
-    ViewSet,
     detail_route,
     list_route,
 )
@@ -56,7 +54,7 @@ from .models import (
 
 # Special Django views
 
-class IndexView(utils_views.CSRFMixin, utils_views.View):
+class IndexView(utils_views.CSRFMixin, utils_views.TemplateView):
     """
     The primary view for OpenSlides using AngularJS.
 
@@ -64,28 +62,20 @@ class IndexView(utils_views.CSRFMixin, utils_views.View):
     You can override it by simply adding a custom 'templates/index.html' file
     to the custom staticfiles directory. See STATICFILES_DIRS in settings.py.
     """
-
-    def get(self, *args, **kwargs):
-        with open(finders.find('templates/index.html')) as f:
-            content = f.read()
-        return HttpResponse(content)
+    template_name = 'templates/index.html'
 
 
-class ProjectorView(utils_views.View):
+class ProjectorView(utils_views.TemplateView):
     """
     The primary view for OpenSlides projector using AngularJS.
 
     The projector container template is 'openslides/core/static/templates/projector-container.html'.
     This container is for controlling the projector resolution.
     """
-
-    def get(self, *args, **kwargs):
-        with open(finders.find('templates/projector-container.html')) as f:
-            content = f.read()
-        return HttpResponse(content)
+    template_name = 'templates/projector-container.html'
 
 
-class RealProjectorView(utils_views.View):
+class RealProjectorView(utils_views.TemplateView):
     """
     The original view without resolutioncontrol for OpenSlides projector using AngularJS.
 
@@ -94,11 +84,7 @@ class RealProjectorView(utils_views.View):
     file to the custom staticfiles directory. See STATICFILES_DIRS in
     settings.py.
     """
-
-    def get(self, *args, **kwargs):
-        with open(finders.find('templates/projector.html')) as f:
-            content = f.read()
-        return HttpResponse(content)
+    template_name = 'templates/projector.html'
 
 
 class WebclientJavaScriptView(utils_views.View):
@@ -107,10 +93,19 @@ class WebclientJavaScriptView(utils_views.View):
     AngularJS app for the requested realm (site or projector). Also code
     for plugins is appended. The result is not uglified.
     """
-    def get(self, *args, **kwargs):
-        angular_modules = []
-        js_files = []
-        realm = kwargs.get('realm')  # Result is 'site' or 'projector'
+    cache = {}  # type: Dict[str, str]
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+        if 'site' not in self.cache:
+            self.init_cache('site')
+        if 'projector' not in self.cache:
+            self.init_cache('projector')
+
+    def init_cache(self, realm: str) -> None:
+        angular_modules = []  # type: List[str]
+        js_files = []  # type: List[str]
         for app_config in apps.get_app_configs():
             # Add the angular app if the module has one.
             if getattr(app_config, 'angular_{}_module'.format(realm), False):
@@ -188,9 +183,12 @@ class WebclientJavaScriptView(utils_views.View):
             """.format(realm=realm, angular_modules=angular_modules, angular_constants=angular_constants, js_files=js_files) +
             """
             }());
-            """)
+            """).replace('\n', '')
+        self.cache[realm] = content
 
-        return HttpResponse(content, content_type='application/javascript')
+    def get(self, *args: Any, **kwargs: Any) -> HttpResponse:
+        realm = cast(str, kwargs.get('realm'))  # Result is 'site' or 'projector'
+        return HttpResponse(self.cache[realm], content_type='application/javascript')
 
 
 # Viewsets for the REST API
@@ -215,8 +213,8 @@ class ProjectorViewSet(ModelViewSet):
         elif self.action in (
             'create', 'update', 'partial_update', 'destroy',
             'activate_elements', 'prune_elements', 'update_elements', 'deactivate_elements', 'clear_elements',
-            'control_view', 'set_resolution', 'set_scroll', 'control_blank', 'broadcast',
-            'set_projectiondefault',
+            'project', 'control_view', 'set_resolution', 'set_scroll', 'control_blank',
+            'broadcast', 'set_projectiondefault',
         ):
             result = (has_perm(self.request.user, 'core.can_see_projector') and
                       has_perm(self.request.user, 'core.can_manage_projector'))
@@ -273,20 +271,39 @@ class ProjectorViewSet(ModelViewSet):
         if not isinstance(request.data, list):
             raise ValidationError({'detail': 'Data must be a list.'})
 
-        projector_instance = self.get_object()
-        projector_config = {}
-        for key, value in projector_instance.config.items():
-            if value.get('stable'):
-                projector_config[key] = value
-        for element in request.data:
+        projector = self.get_object()
+        elements = request.data
+        if not isinstance(elements, list):
+            raise ValidationError({'detail': _('The data has to be a list.')})
+        for element in elements:
+            if not isinstance(element, dict):
+                raise ValidationError({'detail': _('All elements have to be dicts.')})
             if element.get('name') is None:
                 raise ValidationError({'detail': 'Invalid projector element. Name is missing.'})
+        return Response(self.prune(projector, elements))
+
+    def prune(self, projector, elements):
+        """
+        Prunes all non stable elements from the projector and adds the given elements.
+        The elements have to a list of dicts, each gict containing at least a name. This
+        is not validated at this point! Should be done before.
+        Returns the new serialized data.
+        """
+        projector_config = {}
+        for key, value in projector.config.items():
+            if value.get('stable'):
+                projector_config[key] = value
+        for element in elements:
             projector_config[uuid.uuid4().hex] = element
 
-        serializer = self.get_serializer(projector_instance, data={'config': projector_config}, partial=False)
+        serializer = self.get_serializer(projector, data={'config': projector_config}, partial=False)
         serializer.is_valid(raise_exception=True)
         serializer.save()
-        return Response(serializer.data)
+        # reset scroll level
+        if (projector.scroll != 0):
+            projector.scroll = 0
+            projector.save()
+        return serializer.data
 
     @detail_route(methods=['post'])
     def update_elements(self, request, pk):
@@ -368,16 +385,77 @@ class ProjectorViewSet(ModelViewSet):
         entries with stable == True. It expects a POST request to
         /rest/core/projector/<pk>/clear_elements/.
         """
-        projector_instance = self.get_object()
+        projector = self.get_object()
+        return Response(self.clear(projector))
+
+    def clear(self, projector):
         projector_config = {}
-        for key, value in projector_instance.config.items():
+        for key, value in projector.config.items():
             if value.get('stable'):
                 projector_config[key] = value
 
-        serializer = self.get_serializer(projector_instance, data={'config': projector_config}, partial=False)
+        serializer = self.get_serializer(projector, data={'config': projector_config}, partial=False)
         serializer.is_valid(raise_exception=True)
         serializer.save()
-        return Response(serializer.data)
+        return serializer.data
+
+    @list_route(methods=['post'])
+    def project(self, request, *args, **kwargs):
+        """
+        REST API operation. Does a combination of clear_elements and prune_elements:
+        In the most cases when projecting an element it first need to be removed from
+        all projectors where it is projected. In a second step the new element (which
+        may be not given if the element is just deprojected) needs to be projected on
+        a maybe different projector. The request data has to have this scheme:
+        {
+            clear_ids: [<projector id1>, ...],      # May be an empty list
+            prune: {                                # May not be given.
+                id: <projector id>,
+                element: <projector element to add>
+            }
+        }
+        """
+        # The data has to be a dict.
+        if not isinstance(request.data, dict):
+            raise ValidationError({'detail': _('The data has to be a dict.')})
+
+        # Get projector ids to clear
+        clear_projector_ids = request.data.get('clear_ids', [])
+        for id in clear_projector_ids:
+            if not isinstance(id, int):
+                raise ValidationError({'detail': _('The id "{}" has to be int.').format(id)})
+
+        # Get the projector id and validate element to prune. This is optional.
+        prune = request.data.get('prune')
+        if prune is not None:
+            if not isinstance(prune, dict):
+                raise ValidationError({'detail': _('Prune has to be an object.')})
+            prune_projector_id = prune.get('id')
+            if not isinstance(prune_projector_id, int):
+                raise ValidationError({'detail': _('The prune projector id has to be int.')})
+
+            # Get the projector after all clear operations, but check, if it exist.
+            if not Projector.objects.filter(pk=prune_projector_id).exists():
+                raise ValidationError({
+                    'detail': _('The projector with id "{}" does not exist').format(prune_projector_id)})
+
+            prune_element = prune.get('element', {})
+            if not isinstance(prune_element, dict):
+                raise ValidationError({'detail': _('Prune element has to be a dict or not given.')})
+            if prune_element.get('name') is None:
+                raise ValidationError({'detail': 'Invalid projector element. Name is missing.'})
+
+        # First step: Clear all given projectors
+        for projector in Projector.objects.filter(pk__in=clear_projector_ids):
+            self.clear(projector)
+
+        # Second step: optionally prune
+        if prune is not None:
+            # This get is save. We checked that the projector exists above.
+            prune_projector = Projector.objects.get(pk=prune_projector_id)
+            self.prune(prune_projector, [prune_element])
+
+        return Response()
 
     @detail_route(methods=['post'])
     def set_resolution(self, request, pk):
@@ -406,8 +484,8 @@ class ProjectorViewSet(ModelViewSet):
         if not isinstance(request.data['width'], int) or not isinstance(request.data['height'], int):
             raise ValidationError({'detail': 'Data has to be integers.'})
         if (request.data['width'] < 800 or request.data['width'] > 3840 or
-                request.data['height'] < 600 or request.data['height'] > 2160):
-            raise ValidationError({'detail': 'The Resolution have to be between 800x600 and 3840x2160.'})
+                request.data['height'] < 340 or request.data['height'] > 2880):
+            raise ValidationError({'detail': 'The Resolution have to be between 800x340 and 3840x2880.'})
 
         projector_instance = self.get_object()
         projector_instance.width = request.data['width']
@@ -580,37 +658,7 @@ class TagViewSet(ModelViewSet):
         return result
 
 
-class ConfigMetadata(SimpleMetadata):
-    """
-    Custom metadata class to add config info to responses on OPTIONS requests.
-    """
-    def determine_metadata(self, request, view):
-        # Build tree.
-        config_groups = []
-        for config_variable in sorted(config.config_variables.values(), key=attrgetter('weight')):
-            if config_variable.is_hidden():
-                # Skip hidden config variables. Do not even check groups and subgroups.
-                continue
-            if not config_groups or config_groups[-1]['name'] != config_variable.group:
-                # Add new group.
-                config_groups.append(OrderedDict(
-                    name=config_variable.group,
-                    subgroups=[]))
-            if not config_groups[-1]['subgroups'] or config_groups[-1]['subgroups'][-1]['name'] != config_variable.subgroup:
-                # Add new subgroup.
-                config_groups[-1]['subgroups'].append(OrderedDict(
-                    name=config_variable.subgroup,
-                    items=[]))
-            # Add the config variable to the current group and subgroup.
-            config_groups[-1]['subgroups'][-1]['items'].append(config_variable.data)
-
-        # Add tree to metadata.
-        metadata = super().determine_metadata(request, view)
-        metadata['config_groups'] = config_groups
-        return metadata
-
-
-class ConfigViewSet(ViewSet):
+class ConfigViewSet(ModelViewSet):
     """
     API endpoint for the config.
 
@@ -618,7 +666,7 @@ class ConfigViewSet(ViewSet):
     partial_update.
     """
     access_permissions = ConfigAccessPermissions()
-    metadata_class = ConfigMetadata
+    queryset = ConfigStore.objects.all()
 
     def check_view_permissions(self):
         """
@@ -632,34 +680,18 @@ class ConfigViewSet(ViewSet):
             # enabled.
             result = self.request.user.is_authenticated() or anonymous_is_enabled()
         elif self.action in ('partial_update', 'update'):
-            result = has_perm(self.request.user, 'core.can_manage_config')
+            # The user needs 'core.can_manage_logos_and_fonts' for all config values
+            # starting with 'logo' and 'font'. For all other config values th euser needs
+            # the default permissions 'core.can_manage_config'.
+            pk = self.kwargs['pk']
+            if pk.startswith('logo') or pk.startswith('font'):
+                result = has_perm(self.request.user, 'core.can_manage_logos_and_fonts')
+            else:
+                result = has_perm(self.request.user, 'core.can_manage_config')
         else:
             result = False
         return result
 
-    def list(self, request):
-        """
-        Lists all config variables.
-        """
-        collection = Collection(config.get_collection_string())
-        return Response(collection.as_list_for_user(request.user))
-
-    def retrieve(self, request, *args, **kwargs):
-        """
-        Retrieves a config variable.
-        """
-        key = kwargs['pk']
-        collection_element = CollectionElement.from_values(config.get_collection_string(), key)
-        try:
-            content = collection_element.as_dict_for_user(request.user)
-        except ConfigStore.DoesNotExist:
-            raise Http404
-        if content is None:
-            # If content is None, the user has no permissions to see the item.
-            # See ConfigAccessPermissions or rather its parent class.
-            self.permission_denied()
-        return Response(content)
-
     def update(self, request, *args, **kwargs):
         """
         Updates a config variable. Only managers can do this.
@@ -719,6 +751,9 @@ class ChatMessageViewSet(ModelViewSet):
         method so that the request.user can be saved into the model field.
         """
         serializer.save(user=self.request.user)
+        # Send chatter via autoupdate because users without permission
+        # to see users may not have it but can get it now.
+        inform_changed_data([self.request.user])
 
     @list_route(methods=['post'])
     def clear(self, request):
@@ -729,11 +764,11 @@ class ChatMessageViewSet(ModelViewSet):
         chatmessages = ChatMessage.objects.all()
         args = []
         for chatmessage in chatmessages:
-            args.append(chatmessage.get_collection_string())
-            args.append(chatmessage.pk)
+            args.append((chatmessage.get_collection_string(), chatmessage.pk))
         chatmessages.delete()
         # Trigger autoupdate and setup response.
-        inform_deleted_data(*args)
+        if len(args) > 0:
+            inform_deleted_data(args)
         return Response({'detail': _('All chat messages deleted successfully.')})
 
 
@@ -803,65 +838,22 @@ class VersionView(utils_views.APIView):
     http_method_names = ['get']
 
     def get_context_data(self, **context):
-        result = dict(openslides_version=version, plugins=[])
+        Result = TypedDict('Result', {  # noqa
+            'openslides_version': str,
+            'openslides_license': str,
+            'openslides_url': str,
+            'plugins': List[Dict[str, str]]})
+        result = dict(
+            openslides_version=version,
+            openslides_license=license,
+            openslides_url=url,
+            plugins=[])  # type: Result
         # Versions of plugins.
         for plugin in settings.INSTALLED_PLUGINS:
             result['plugins'].append({
                 'verbose_name': get_plugin_verbose_name(plugin),
                 'description': get_plugin_description(plugin),
-                'version': get_plugin_version(plugin)})
+                'version': get_plugin_version(plugin),
+                'license': get_plugin_license(plugin),
+                'url': get_plugin_url(plugin)})
         return result
-
-
-class MediaEncoder(utils_views.APIView):
-    """
-    MediaEncoder is a class based view to prepare encoded media for pdfMake
-    """
-    http_method_names = ['post']
-
-    def post(self, request, *args, **kwargs):
-        """
-        Encode_image is used in the context of PDF-Generation
-        Takes an array of IMG.src - Paths
-        Retrieves the according images
-        Encodes the images to BASE64
-        Puts it into a key-value structure
-
-        {
-            "images": {
-                "media/file/ubuntu.png":"$ENCODED_IMAGE"
-            }
-        }
-
-        :param request:
-        :return: Response of the resulting dictionary
-
-        Calling e.g.
-        $.ajax({ type: "POST", url: "/motions/encode_images/",
-                data: JSON.stringify(["$FILEPATH"]),
-                success: function(data){ console.log(data); },
-                dataType: 'application/json' });
-        """
-        body_unicode = request.body.decode('utf-8')
-        file_paths = json.loads(body_unicode)
-        images = {file_path: self.encode_image_from(file_path) for file_path in file_paths}
-        return Response({
-            "images": images
-        })
-
-    def encode_image_from(self, file_path):
-        """
-        Returns the BASE64 encoded version of an image-file for a given path
-        :param file_path:
-        :return:
-        """
-        path = os.path.join(settings.MEDIA_ROOT, 'file', os.path.basename(file_path))
-        try:
-            with open(path, "rb") as file:
-                string_representation = "data:image/{};base64,{}".format(os.path.splitext(file_path)[1][1:],
-                                                                         base64.b64encode(file.read()).decode())
-        except Exception:
-            # If any error occurs ignore it and return an empty string
-            return ""
-        else:
-            return string_representation
