@@ -1,6 +1,8 @@
+from typing import Any, Dict  # noqa
+
 from django.conf import settings
 from django.contrib.contenttypes.fields import GenericRelation
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import IntegrityError, models, transaction
 from django.db.models import Max
 from django.utils import formats, timezone
@@ -10,7 +12,7 @@ from jsonfield import JSONField
 
 from openslides.agenda.models import Item
 from openslides.core.config import config
-from openslides.core.models import Tag
+from openslides.core.models import Projector, Tag
 from openslides.mediafiles.models import Mediafile
 from openslides.poll.models import (
     BaseOption,
@@ -167,7 +169,7 @@ class Motion(RESTModelMixin, models.Model):
 
     comments = JSONField(null=True)
     """
-    Configurable fields for comments. Contains a list of strings.
+    Configurable fields for comments.
     """
 
     # In theory there could be one then more agenda_item. But we support only
@@ -180,7 +182,8 @@ class Motion(RESTModelMixin, models.Model):
             ('can_see', 'Can see motions'),
             ('can_create', 'Can create motions'),
             ('can_support', 'Can support motions'),
-            ('can_see_and_manage_comments', 'Can see and manage comments'),
+            ('can_see_comments', 'Can see comments'),
+            ('can_manage_comments', 'Can manage comments'),
             ('can_manage', 'Can manage motions'),
         )
         ordering = ('identifier', )
@@ -231,13 +234,19 @@ class Motion(RESTModelMixin, models.Model):
             try:
                 # Always skip autoupdate. Maybe we run it later in this method.
                 with transaction.atomic():
-                    super(Motion, self).save(skip_autoupdate=True, *args, **kwargs)
+                    super(Motion, self).save(skip_autoupdate=True, *args, **kwargs)  # type: ignore
             except IntegrityError:
-                # Identifier is already used. Calculate a new one and try again.
-                self.identifier_number, self.identifier = self.increment_identifier_number(
-                    self.identifier_number,
-                    self._identifier_prefix,
-                )
+                # Identifier is already used.
+                if hasattr(self, '_identifier_prefix'):
+                    # Calculate a new one and try again.
+                    self.identifier_number, self.identifier = self.increment_identifier_number(
+                        self.identifier_number,
+                        self._identifier_prefix,
+                    )
+                else:
+                    # Do not calculate a new one but reraise the IntegrityError.
+                    # The error is caught in the category sort view.
+                    raise
             else:
                 # Save was successful. End loop.
                 break
@@ -294,6 +303,17 @@ class Motion(RESTModelMixin, models.Model):
         if not skip_autoupdate:
             inform_changed_data(self)
 
+    def delete(self, skip_autoupdate=False, *args, **kwargs):
+        """
+        Customized method to delete a motion. Ensures that a respective
+        motion projector element is disabled.
+        """
+        Projector.remove_any(
+            skip_autoupdate=skip_autoupdate,
+            name='motions/motion',
+            id=self.pk)
+        return super().delete(skip_autoupdate=skip_autoupdate, *args, **kwargs)  # type: ignore
+
     def version_data_changed(self, version):
         """
         Compare the version with the last version of the motion.
@@ -321,21 +341,6 @@ class Motion(RESTModelMixin, models.Model):
             # Do not set an identifier.
             return
 
-        # The motion is an amendment.
-        elif self.is_amendment():
-            motions = self.parent.amendments.all()
-
-        # The motions should be counted per category.
-        elif config['motions_identifier'] == 'per_category':
-            motions = Motion.objects.filter(category=self.category)
-
-        # The motions should be counted over all.
-        else:
-            motions = Motion.objects.all()
-
-        # Get biggest number.
-        number = motions.aggregate(Max('identifier_number'))['identifier_number__max'] or 0
-
         # If MOTION_IDENTIFIER_WITHOUT_BLANKS is set, don't use blanks when building identifier.
         without_blank = hasattr(settings, 'MOTION_IDENTIFIER_WITHOUT_BLANKS') and settings.MOTION_IDENTIFIER_WITHOUT_BLANKS
 
@@ -355,19 +360,42 @@ class Motion(RESTModelMixin, models.Model):
                 prefix = '%s ' % self.category.prefix
         self._identifier_prefix = prefix
 
+        # Use the already assigned identifier_number, if the motion has one.
+        # Else get the biggest number.
+        if self.identifier_number is not None:
+            number = self.identifier_number
+            initial_increment = False
+        else:
+            # Find all motions that should be included in the calculations.
+            if self.is_amendment():
+                motions = self.parent.amendments.all()
+            # The motions should be counted per category.
+            elif config['motions_identifier'] == 'per_category':
+                motions = Motion.objects.filter(category=self.category)
+            # The motions should be counted over all.
+            else:
+                motions = Motion.objects.all()
+
+            number = motions.aggregate(Max('identifier_number'))['identifier_number__max'] or 0
+            initial_increment = True
+
         # Calculate new identifier.
-        number, identifier = self.increment_identifier_number(number, prefix)
+        number, identifier = self.increment_identifier_number(
+            number,
+            prefix,
+            initial_increment=initial_increment)
 
         # Set identifier and identifier_number.
         self.identifier = identifier
         self.identifier_number = number
 
-    def increment_identifier_number(self, number, prefix):
+    def increment_identifier_number(self, number, prefix, initial_increment=True):
         """
         Helper method. It increments the number until a free identifier
         number is found. Returns new number and identifier.
         """
-        number += 1
+        if initial_increment:
+            number += 1
         identifier = '%s%s' % (prefix, self.extend_identifier_number(number))
         while Motion.objects.filter(identifier=identifier).exists():
             number += 1
@@ -528,15 +556,16 @@ class Motion(RESTModelMixin, models.Model):
         """
         return user in self.supporters.all()
 
-    def create_poll(self):
+    def create_poll(self, skip_autoupdate=False):
         """
         Create a new poll for this motion.
 
         Return the new poll object.
         """
         if self.state.allow_create_poll:
-            poll = MotionPoll.objects.create(motion=self)
-            poll.set_options()
+            poll = MotionPoll(motion=self)
+            poll.save(skip_autoupdate=skip_autoupdate)
+            poll.set_options(skip_autoupdate=skip_autoupdate)
             return poll
         else:
             raise WorkflowError('You can not create a poll in state %s.' % self.state.name)
@@ -564,13 +593,15 @@ class Motion(RESTModelMixin, models.Model):
 
     def reset_state(self, workflow=None):
         """
-        Set the state to the default state.
+        Set the state to the default state. If an identifier was set
+        automatically, reset the identifier and identifier_number.
 
         'workflow' can be a workflow, an id of a workflow or None.
 
         If the motion is new and workflow is None, it chooses the default
         workflow from config.
         """
+
         if type(workflow) is int:
             workflow = Workflow.objects.get(pk=workflow)
 
@@ -600,6 +631,11 @@ class Motion(RESTModelMixin, models.Model):
         if self.recommendation is not None:
             self.set_state(self.recommendation)
 
+    """
+    Container for runtime information for agenda app (on create or update of this instance).
+    """
+    agenda_item_update_information = {}  # type: Dict[str, Any]
+
     def get_agenda_title(self):
         """
         Return a simple title string for the agenda.
@@ -618,9 +654,9 @@ class Motion(RESTModelMixin, models.Model):
         Note: It has to be the same return value like in JavaScript.
         """
         if self.identifier:
-            string = '%s (%s %s)' % (self.title, _(self._meta.verbose_name), self.identifier)
+            string = '%s %s' % (_(self._meta.verbose_name), self.identifier)
         else:
-            string = '%s (%s)' % (self.title, _(self._meta.verbose_name))
+            string = '%s (%s)' % (_(self._meta.verbose_name), self.title)
         return string
 
     @property
@@ -751,7 +787,10 @@ class MotionChangeRecommendation(RESTModelMixin, models.Model):
     """If true, this change recommendation has been rejected"""
 
     type = models.PositiveIntegerField(default=0)
-    """Replacement (0), Insertion (1), Deletion (2)"""
+    """Replacement (0), Insertion (1), Deletion (2), Other (3)"""
+
+    other_description = models.TextField(blank=True)
+    """The description text for type 'other'"""
 
     line_from = models.PositiveIntegerField()
     """The number or the first affected line"""
@@ -771,6 +810,25 @@ class MotionChangeRecommendation(RESTModelMixin, models.Model):
     creation_time = models.DateTimeField(auto_now=True)
     """Time when the change recommendation was saved."""
 
+    def collides_with_other_recommendation(self, recommendations):
+        for recommendation in recommendations:
+            if (not (self.line_from < recommendation.line_from and self.line_to <= recommendation.line_from) and
+                    not (self.line_from >= recommendation.line_to and self.line_to > recommendation.line_to)):
+                return True
+
+        return False
+
+    def save(self, *args, **kwargs):
+        recommendations = (MotionChangeRecommendation.objects
+                           .filter(motion_version=self.motion_version)
+                           .exclude(pk=self.pk))
+
+        if self.collides_with_other_recommendation(recommendations):
+            raise ValidationError('The recommendation collides with an existing one (line %s - %s).' %
+                                  (self.line_from, self.line_to))
+
+        return super().save(*args, **kwargs)
+
     class Meta:
         default_permissions = ()
 
@@ -834,6 +892,22 @@ class MotionBlock(RESTModelMixin, models.Model):
     def __str__(self):
         return self.title
 
+    def delete(self, skip_autoupdate=False, *args, **kwargs):
+        """
+        Customized method to delete a motion block. Ensures that a respective
+        motion block projector element is disabled.
+        """
+        Projector.remove_any(
+            skip_autoupdate=skip_autoupdate,
+            name='motions/motion-block',
+            id=self.pk)
+        return super().delete(skip_autoupdate=skip_autoupdate, *args, **kwargs)  # type: ignore
+
+    """
+    Container for runtime information for agenda app (on create or update of this instance).
+    """
+    agenda_item_update_information = {}  # type: Dict[str, Any]
+
     @property
     def agenda_item(self):
         """
@@ -947,7 +1021,9 @@ class MotionOption(RESTModelMixin, BaseOption):
         return self.poll.motion
 
 
-class MotionPoll(RESTModelMixin, CollectDefaultVotesMixin, BasePoll):
+# TODO: remove the type-ignoring in the next line, after this is solved:
+#       https://github.com/python/mypy/issues/3855
+class MotionPoll(RESTModelMixin, CollectDefaultVotesMixin, BasePoll):  # type: ignore
     """The Class to saves the vote result for a motion poll."""
 
     motion = models.ForeignKey(
@@ -971,11 +1047,11 @@ class MotionPoll(RESTModelMixin, CollectDefaultVotesMixin, BasePoll):
         """
         return 'MotionPoll for motion %s' % self.motion
 
-    def set_options(self):
+    def set_options(self, skip_autoupdate=False):
         """Create the option class for this poll."""
         # TODO: maybe it is possible with .create() to call this without poll=self
         #       or call this in save()
-        self.get_option_class()(poll=self).save()
+        self.get_option_class()(poll=self).save(skip_autoupdate=skip_autoupdate)
 
     def get_percent_base_choice(self):
         return config['motions_poll_100_percent_base']
