@@ -1,8 +1,10 @@
+from typing import Dict  # noqa
+
 from django.db import transaction
 from django.utils.translation import ugettext as _
 
-from openslides.poll.serializers import default_votes_validator
-from openslides.utils.rest_api import (
+from ..poll.serializers import default_votes_validator
+from ..utils.rest_api import (
     CharField,
     DictField,
     Field,
@@ -12,8 +14,7 @@ from openslides.utils.rest_api import (
     SerializerMethodField,
     ValidationError,
 )
-from openslides.utils.validate import validate_html
-
+from ..utils.validate import validate_html
 from .models import (
     Category,
     Motion,
@@ -48,9 +49,25 @@ class MotionBlockSerializer(ModelSerializer):
     """
     Serializer for motion.models.Category objects.
     """
+    agenda_type = IntegerField(write_only=True, required=False, min_value=1, max_value=2)
+    agenda_parent_id = IntegerField(write_only=True, required=False, min_value=1)
+
     class Meta:
         model = MotionBlock
-        fields = ('id', 'title', 'agenda_item_id',)
+        fields = ('id', 'title', 'agenda_item_id', 'agenda_type', 'agenda_parent_id',)
+
+    def create(self, validated_data):
+        """
+        Customized create method. Set information about related agenda item
+        into agenda_item_update_information container.
+        """
+        agenda_type = validated_data.pop('agenda_type', None)
+        agenda_parent_id = validated_data.pop('agenda_parent_id', None)
+        motion_block = MotionBlock(**validated_data)
+        motion_block.agenda_item_update_information['type'] = agenda_type
+        motion_block.agenda_item_update_information['parent_id'] = agenda_parent_id
+        motion_block.save()
+        return motion_block
 
 
 class StateSerializer(ModelSerializer):
@@ -104,11 +121,15 @@ class MotionCommentsJSONSerializerField(Field):
         """
         Checks that data is a list of strings.
         """
-        if type(data) is not list:
-            raise ValidationError({'detail': 'Data must be an array.'})
-        for element in data:
-            if type(element) is not str:
-                raise ValidationError({'detail': 'Data must be an array of strings.'})
+        if type(data) is not dict:
+            raise ValidationError({'detail': 'Data must be a dict.'})
+        for id, comment in data.items():
+            try:
+                id = int(id)
+            except ValueError:
+                raise ValidationError({'detail': 'Id must be an int.'})
+            if type(comment) is not str:
+                raise ValidationError({'detail': 'Comment must be a string.'})
         return data
 
 
@@ -158,7 +179,7 @@ class MotionPollSerializer(ModelSerializer):
 
     def __init__(self, *args, **kwargs):
         # The following dictionary is just a cache for several votes.
-        self._votes_dicts = {}
+        self._votes_dicts = {}  # type: Dict[int, Dict[int, int]]
         return super().__init__(*args, **kwargs)
 
     def get_yes(self, obj):
@@ -253,13 +274,18 @@ class MotionChangeRecommendationSerializer(ModelSerializer):
             'motion_version',
             'rejected',
             'type',
+            'other_description',
             'line_from',
             'line_to',
             'text',
             'creation_time',)
 
+    def is_title_cr(self, data):
+        return int(data['line_from']) == 0 and int(data['line_to']) == 0
+
     def validate(self, data):
-        if 'text' in data:
+        # Change recommendations for titles are stored as plain-text, thus they don't need to be html-escaped
+        if 'text' in data and not self.is_title_cr(data):
             data['text'] = validate_html(data['text'])
         return data
 
@@ -282,6 +308,8 @@ class MotionSerializer(ModelSerializer):
         required=False,
         validators=[validate_workflow_field],
         write_only=True)
+    agenda_type = IntegerField(write_only=True, required=False, min_value=1, max_value=2)
+    agenda_parent_id = IntegerField(write_only=True, required=False, min_value=1)
 
     class Meta:
         model = Motion
@@ -308,6 +336,8 @@ class MotionSerializer(ModelSerializer):
             'attachments',
             'polls',
             'agenda_item_id',
+            'agenda_type',
+            'agenda_parent_id',
             'log_messages',)
         read_only_fields = ('state', 'recommendation',)  # Some other fields are also read_only. See definitions above.
 
@@ -316,9 +346,9 @@ class MotionSerializer(ModelSerializer):
             data['text'] = validate_html(data['text'])
         if 'reason' in data:
             data['reason'] = validate_html(data['reason'])
-        validated_comments = []
-        for comment in data.get('comments', []):
-            validated_comments.append(validate_html(comment))
+        validated_comments = dict()
+        for id, comment in data.get('comments', {}).items():
+            validated_comments[id] = validate_html(comment)
         data['comments'] = validated_comments
         return data
 
@@ -326,6 +356,9 @@ class MotionSerializer(ModelSerializer):
     def create(self, validated_data):
         """
         Customized method to create a new motion from some data.
+
+        Set also information about related agenda item into
+        agenda_item_update_information container.
         """
         motion = Motion()
         motion.title = validated_data['title']
@@ -338,6 +371,8 @@ class MotionSerializer(ModelSerializer):
         motion.comments = validated_data.get('comments')
         motion.parent = validated_data.get('parent')
         motion.reset_state(validated_data.get('workflow_id'))
+        motion.agenda_item_update_information['type'] = validated_data.get('agenda_type')
+        motion.agenda_item_update_information['parent_id'] = validated_data.get('agenda_parent_id')
         motion.save()
         if validated_data.get('submitters'):
             motion.submitters.add(*validated_data['submitters'])
