@@ -1,8 +1,8 @@
-import base64
 import re
+from typing import Optional  # noqa
 
 from django.conf import settings
-from django.contrib.staticfiles import finders
+from django.core.exceptions import ValidationError as DjangoValidationError
 from django.db import IntegrityError, transaction
 from django.http import Http404
 from django.utils.translation import ugettext as _
@@ -22,7 +22,7 @@ from ..utils.rest_api import (
     ValidationError,
     detail_route,
 )
-from ..utils.views import APIView
+from ..utils.views import BinaryTemplateView
 from .access_permissions import (
     CategoryAccessPermissions,
     MotionAccessPermissions,
@@ -63,16 +63,17 @@ class MotionViewSet(ModelViewSet):
         """
         if self.action in ('list', 'retrieve'):
             result = self.get_access_permissions().check_permissions(self.request.user)
-        elif self.action in ('metadata', 'partial_update', 'update'):
+        elif self.action in ('metadata', 'partial_update', 'update', 'destroy'):
             result = has_perm(self.request.user, 'motions.can_see')
-            # For partial_update and update requests the rest of the check is
+            # For partial_update, update and delete requests the rest of the check is
             # done in the update method. See below.
         elif self.action == 'create':
             result = (has_perm(self.request.user, 'motions.can_see') and
                       has_perm(self.request.user, 'motions.can_create') and
                       (not config['motions_stop_submitting'] or
                        has_perm(self.request.user, 'motions.can_manage')))
-        elif self.action in ('destroy', 'manage_version', 'set_state', 'set_recommendation', 'create_poll'):
+        elif self.action in ('manage_version', 'set_state', 'set_recommendation',
+                             'follow_recommendation', 'create_poll'):
             result = (has_perm(self.request.user, 'motions.can_see') and
                       has_perm(self.request.user, 'motions.can_manage'))
         elif self.action == 'support':
@@ -82,6 +83,19 @@ class MotionViewSet(ModelViewSet):
             result = False
         return result
 
+    def destroy(self, request, *args, **kwargs):
+        """
+        Destroy is allowed if the user has manage permissions, or he is the submitter and
+        the current state allows to edit the motion.
+        """
+        motion = self.get_object()
+
+        if not ((has_perm(request.user, 'motions.can_manage') or
+                motion.is_submitter(request.user) and motion.state.allow_submitter_edit)):
+            self.permission_denied(request)
+
+        return super().destroy(request, *args, **kwargs)
+
     def create(self, request, *args, **kwargs):
         """
         Customized view endpoint to create a new motion.
@@ -91,7 +105,7 @@ class MotionViewSet(ModelViewSet):
             try:
                 parent_motion = CollectionElement.from_values(
                     Motion.get_collection_string(),
-                    request.data['parent_id'])
+                    request.data['parent_id'])  # type: Optional[CollectionElement]
             except Motion.DoesNotExist:
                 raise ValidationError({'detail': _('The parent motion does not exist.')})
         else:
@@ -122,7 +136,8 @@ class MotionViewSet(ModelViewSet):
                     del request.data[key]
 
         # Check permission to send comment data.
-        if not has_perm(request.user, 'motions.can_see_and_manage_comments'):
+        if (not has_perm(request.user, 'motions.can_see_comments') or
+                not has_perm(request.user, 'motions.can_manage_comments')):
             try:
                 # Ignore comments data if user is not allowed to send comments.
                 del request.data['comments']
@@ -137,6 +152,13 @@ class MotionViewSet(ModelViewSet):
 
         # Write the log message and initiate response.
         motion.write_log([ugettext_noop('Motion created')], request.user)
+
+        # Send new submitters and supporters via autoupdate because users
+        # without permission to see users may not have them but can get it now.
+        new_users = list(motion.submitters.all())
+        new_users.extend(motion.supporters.all())
+        inform_changed_data(new_users)
+
         headers = self.get_success_headers(serializer.data)
         return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
 
@@ -155,7 +177,8 @@ class MotionViewSet(ModelViewSet):
         # Check permissions.
         if (not has_perm(request.user, 'motions.can_manage') and
                 not (motion.is_submitter(request.user) and motion.state.allow_submitter_edit) and
-                not has_perm(request.user, 'motions.can_see_and_manage_comments')):
+                not (has_perm(request.user, 'motions.can_see_comments') and
+                     has_perm(request.user, 'motions.can_manage_comments'))):
             self.permission_denied(request)
 
         # Check permission to send only some data.
@@ -176,12 +199,49 @@ class MotionViewSet(ModelViewSet):
             for key in keys:
                 if key not in whitelist:
                     del request.data[key]
-        if not has_perm(request.user, 'motions.can_see_and_manage_comments'):
-            try:
+
+        # Check comments
+        # "normal" comments only can be changed if the user has can_see_comments and
+        # can_manage_comments.
+        # "special" comments (for state and recommendation) can only be changed, if
+        # the user has the can_manage permission
+        if 'comments' in request.data:
+            request_comments = {}  # Here, all valid comments are saved.
+            for id, value in request.data['comments'].items():
+                field = config['motions_comments'].get(id)
+                if field:
+                    is_special_comment = field.get('forRecommendation') or field.get('forState')
+                    if (is_special_comment and has_perm(request.user, 'motions.can_manage')) or (
+                        not is_special_comment and has_perm(request.user, 'motions.can_see_comments') and
+                            has_perm(request.user, 'motions.can_manage_comments')):
+                        # The user has the required permission for at least one case
+                        # Save the comment!
+                        request_comments[id] = value
+
+            # two possibilities here: Either the comments dict is empty: then delete it, so
+            # the serializer will skip it. If we leave it empty, everything will be deleted :(
+            # Second, just special or normal comments are in the comments dict. Fill the original
+            # data, so it won't be delete.
+
+            if len(request_comments) == 0:
                 del request.data['comments']
-            except KeyError:
-                # No comments here. Just do nothing.
-                pass
+            else:
+                if motion.comments:
+                    for id, value in motion.comments.items():
+                        if id not in request_comments:
+                            # populate missing comments with original ones.
+                            request_comments[id] = value
+                request.data['comments'] = request_comments
+
+        # get changed comment fields
+        changed_comment_fields = []
+        comments = request.data.get('comments', {})
+        for id, value in comments.items():
+            if not motion.comments or motion.comments.get(id) != value:
+                field = config['motions_comments'].get(id)
+                if field:
+                    name = field['name']
+                    changed_comment_fields.append(name)
 
         # Validate data and update motion.
         serializer = self.get_serializer(
@@ -198,6 +258,18 @@ class MotionViewSet(ModelViewSet):
                 not has_perm(request.user, 'motions.can_manage')):
             updated_motion.supporters.clear()
             updated_motion.write_log([ugettext_noop('All supporters removed')], request.user)
+
+        if len(changed_comment_fields) > 0:
+            updated_motion.write_log(
+                [ugettext_noop('Comment {} updated').format(', '.join(changed_comment_fields))],
+                request.user)
+
+        # Send new submitters and supporters via autoupdate because users
+        # without permission to see users may not have them but can get it now.
+        new_users = list(updated_motion.submitters.all())
+        new_users.extend(updated_motion.supporters.all())
+        inform_changed_data(new_users)
+
         return Response(serializer.data)
 
     @detail_route(methods=['put', 'delete'])
@@ -265,6 +337,9 @@ class MotionViewSet(ModelViewSet):
                 raise ValidationError({'detail': _('You can not support this motion.')})
             motion.supporters.add(request.user)
             motion.write_log([ugettext_noop('Motion supported')], request.user)
+            # Send new supporter via autoupdate because users without permission
+            # to see users may not have it but can get it now.
+            inform_changed_data([request.user])
             message = _('You have supported this motion successfully.')
         else:
             # Unsupport motion.
@@ -306,13 +381,15 @@ class MotionViewSet(ModelViewSet):
             motion.reset_state()
 
         # Save motion.
-        motion.save(update_fields=['state', 'identifier', 'identifier_number'])
+        motion.save(update_fields=['state', 'identifier', 'identifier_number'], skip_autoupdate=True)
         message = _('The state of the motion was set to %s.') % motion.state.name
 
         # Write the log message and initiate response.
         motion.write_log(
             message_list=[ugettext_noop('State set to'), ' ', motion.state.name],
-            person=request.user)
+            person=request.user,
+            skip_autoupdate=True)
+        inform_changed_data(motion)
         return Response({'detail': message})
 
     @detail_route(methods=['put'])
@@ -355,6 +432,37 @@ class MotionViewSet(ModelViewSet):
             person=request.user)
         return Response({'detail': message})
 
+    @detail_route(methods=['post'])
+    def follow_recommendation(self, request, pk=None):
+        motion = self.get_object()
+        if motion.recommendation is None:
+            raise ValidationError({'detail': _('Cannot set an empty recommendation.')})
+
+        # Set state.
+        motion.set_state(motion.recommendation)
+
+        # Set the special state comment.
+        extension = request.data.get('recommendationExtension')
+        if extension is not None:
+            # Find the special "state" comment field.
+            for id, field in config['motions_comments'].items():
+                if isinstance(field, dict) and 'forState' in field and field['forState'] is True:
+                    motion.comments[id] = extension
+                    break
+
+        # Save and write log.
+        motion.save(
+            update_fields=['state', 'identifier', 'identifier_number', 'comments'],
+            skip_autoupdate=True)
+        motion.write_log(
+            message_list=[ugettext_noop('State set to'), ' ', motion.state.name],
+            person=request.user,
+            skip_autoupdate=True)
+
+        # Now send all changes to the clients.
+        inform_changed_data(motion)
+        return Response({'detail': 'Recommendation followed successfully.'})
+
     @detail_route(methods=['post'])
     def create_poll(self, request, pk=None):
         """
@@ -365,11 +473,15 @@ class MotionViewSet(ModelViewSet):
             raise ValidationError({'detail': 'You can not create a poll in this motion state.'})
         try:
             with transaction.atomic():
-                motion.create_poll()
+                poll = motion.create_poll(skip_autoupdate=True)
         except WorkflowError as e:
             raise ValidationError({'detail': e})
-        motion.write_log([ugettext_noop('Vote created')], request.user)
-        return Response({'detail': _('Vote created successfully.')})
+        motion.write_log([ugettext_noop('Vote created')], request.user, skip_autoupdate=True)
+
+        inform_changed_data(motion)
+        return Response({
+            'detail': _('Vote created successfully.'),
+            'createdPollId': poll.pk})
 
 
 class MotionPollViewSet(UpdateModelMixin, DestroyModelMixin, GenericViewSet):
@@ -432,6 +544,15 @@ class MotionChangeRecommendationViewSet(ModelViewSet):
             result = False
         return result
 
+    def create(self, request, *args, **kwargs):
+        """
+        Creating a Change Recommendation, custom exception handling
+        """
+        try:
+            return super().create(request, *args, **kwargs)
+        except DjangoValidationError as err:
+            return Response({'detail': err.message}, status=400)
+
 
 class CategoryViewSet(ModelViewSet):
     """
@@ -519,11 +640,18 @@ class CategoryViewSet(ModelViewSet):
                 # Remove old identifiers
                 for motion in motions:
                     motion.identifier = None
-                    motion.skip_autoupdate = True  # This line is to skip agenda item autoupdate. See agenda/signals.py.
+                    # This line is to skip agenda item autoupdate. See agenda/signals.py.
+                    motion.agenda_item_update_information['skip_autoupdate'] = True
                     motion.save(skip_autoupdate=True)
 
                 # Set new identifers and change identifiers of amendments.
                 for obj in motions_to_be_sorted:
+                    if Motion.objects.filter(identifier=obj['new_identifier']).exists():
+                        raise ValidationError({
+                            'detail':
+                            _('Numbering aborted because the motion identifier "%s" already exists outside of this category.') %
+                            obj['new_identifier']
+                        })
                     motion = obj['motion']
                     motion.identifier = obj['new_identifier']
                     motion.identifier_number = obj['number']
@@ -532,13 +660,14 @@ class CategoryViewSet(ModelViewSet):
                     instances.append(motion.agenda_item)
                     # Change identifiers of amendments.
                     for child in motion.get_amendments_deep():
-                        if child.identifier.startswith(obj['old_identifier']):
+                        if child.identifier and child.identifier.startswith(obj['old_identifier']):
                             child.identifier = re.sub(
                                 obj['old_identifier'],
                                 obj['new_identifier'],
                                 child.identifier,
                                 count=1)
-                            child.skip_autoupdate = True  # This line is to skip agenda item autoupdate. See agenda/signals.py.
+                            # This line is to skip agenda item autoupdate. See agenda/signals.py.
+                            child.agenda_item_update_information['skip_autoupdate'] = True
                             child.save(skip_autoupdate=True)
                             instances.append(child)
                             instances.append(child.agenda_item)
@@ -628,15 +757,10 @@ class WorkflowViewSet(ModelViewSet):
         return result
 
 
-# Special API views
+# Special views
 
-class MotionDocxTemplateView(APIView):
+class MotionDocxTemplateView(BinaryTemplateView):
     """
     Returns the template for motions docx export
     """
-    http_method_names = ['get']
-
-    def get_context_data(self, **context):
-        with open(finders.find('templates/docx/motions.docx'), "rb") as file:
-            response = base64.b64encode(file.read())
-        return response
+    template_name = 'templates/docx/motions.docx'
