@@ -128,23 +128,29 @@ angular.module('OpenSlidesApp.users', [])
 ])
 
 .factory('Group', [
-    '$http',
     'DS',
-    function($http, DS) {
+    function(DS) {
+        var name = 'users/group';
         var permissions;
         return DS.defineResource({
-            name: 'users/group',
-            permissions: permissions,
-            // TODO (Issue 2862): Do not query the permissions from server. They should be included
-            // in the startup data. Then remove 'permission' injection from group list controller.
-            getPermissions: function() {
-                if (!this.permissions) {
-                    this.permissions = $http({ 'method': 'OPTIONS', 'url': '/rest/users/group/' })
-                        .then(function(result) {
-                            return result.data.actions.POST.permissions.choices;
-                        });
+            name: name,
+        });
+    }
+])
+
+.factory('PersonalNote', [
+    'DS',
+    function (DS) {
+        var name = 'users/personal-note';
+        return DS.defineResource({
+            name: name,
+            relations: {
+                hasOne: {
+                    'users/user': {
+                        localField: 'user',
+                        localKey: 'user_id',
+                    }
                 }
-                return this.permissions;
             }
         });
     }
@@ -153,10 +159,10 @@ angular.module('OpenSlidesApp.users', [])
 .run([
     'User',
     'Group',
+    'PersonalNote',
     function(User, Group) {}
 ])
 
-
 // Mark strings for translation in JavaScript.
 .config([
     'gettext',
