@@ -76,6 +76,10 @@ angular.module('OpenSlidesApp.users.site', [
             url: '/import',
             controller: 'UserImportCtrl',
         })
+        .state('users.user.presence', {
+            url: '/presence',
+            controller: 'UserPresenceCtrl',
+        })
         // groups
         .state('users.group', {
             url: '/groups',
@@ -85,13 +89,7 @@ angular.module('OpenSlidesApp.users.site', [
                 title: gettext('Groups'),
             },
         })
-        .state('users.group.list', {
-            resolve: {
-                permissions: function(Group) {
-                    return Group.getPermissions();
-                }
-            }
-        })
+        .state('users.group.list', {})
         .state('login', {
             template: null,
             url: '/login',
@@ -99,14 +97,14 @@ angular.module('OpenSlidesApp.users.site', [
                 guest_enabled: false,
                 msg: null,
             },
-            onEnter: ['$state', '$stateParams', 'ngDialog', function($state, $stateParams, ngDialog) {
-                ngDialog.open({
+            onEnter: ['$state', '$stateParams', 'ngDialog', 'LoginDialog', function($state, $stateParams, ngDialog, LoginDialog) {
+                LoginDialog.id = ngDialog.open({
                     template: 'static/templates/core/login-form.html',
                     controller: 'LoginFormCtrl',
                     showClose: $stateParams.guest_enabled,
                     closeByEscape: $stateParams.guest_enabled,
                     closeByDocument: $stateParams.guest_enabled,
-                });
+                }).id;
             }],
             data: {
                 title: 'Login',
@@ -115,6 +113,8 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
+.value('LoginDialog', {})
+
 /*
  * Directive to check for permissions
  *
@@ -203,6 +203,53 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
+.factory('PersonalNoteManager', [
+    'PersonalNote',
+    'operator',
+    function (PersonalNote, operator) {
+        var _getPersonalNoteObject = function (resourceName) {
+            var personalNote = _.find(PersonalNote.getAll(), function (pn) {
+                return pn.user_id === operator.user.id;
+            });
+            if (!personalNote) {
+                personalNote = {
+                    notes: {},
+                };
+            }
+            if (!personalNote.notes[resourceName]) {
+                personalNote.notes[resourceName] = {};
+            }
+            return personalNote;
+        };
+        var get = function (resourceName, id) {
+            return _getPersonalNoteObject(resourceName).notes[resourceName][id];
+        };
+        var save = function (resourceName, id, note) {
+            var personalNote = _getPersonalNoteObject(resourceName);
+            personalNote.notes[resourceName][id] = note;
+            if (personalNote.id) {
+                return PersonalNote.save(personalNote);
+            } else {
+                return PersonalNote.create(personalNote);
+            }
+        };
+        return {
+            getNote: function (obj) {
+                if (typeof obj.getResourceName === 'undefined') {
+                    throw 'The Object has to be a js data model!';
+                }
+                return get(obj.getResourceName(), obj.id);
+            },
+            saveNote: function (obj, note) {
+                if (typeof obj.getResourceName === 'undefined') {
+                    throw 'The Object has to be a js data model!';
+                }
+                return save(obj.getResourceName(), obj.id, note);
+            },
+        };
+    }
+])
+
 // Service for generic assignment form (create and update)
 .factory('UserForm', [
     '$http',
@@ -259,6 +306,13 @@ angular.module('OpenSlidesApp.users.site', [
                         }
                     ]
                 },
+                {
+                    key: 'email',
+                    type: 'input',
+                    templateOptions: {
+                        label: gettextCatalog.getString('Email')
+                    },
+                },
                 {
                     className: "row",
                     fieldGroup: [
@@ -292,9 +346,8 @@ angular.module('OpenSlidesApp.users.site', [
                     type: 'select-multiple',
                     templateOptions: {
                         label: gettextCatalog.getString('Groups'),
-                        options: Group.getAll(),
-                        ngOptions: 'option.id as option.name | translate for option in to.options | ' +
-                                   'filter: {id: "!1"}',
+                        options: Group.filter({where: {id: {'>': 1}}}),
+                        ngOptions: "option.id as option.name | translate for option in to.options | orderBy: 'id'",
                         placeholder: gettextCatalog.getString('Select or search a group ...')
                     }
                 },
@@ -390,7 +443,7 @@ angular.module('OpenSlidesApp.users.site', [
     function (gettextCatalog, Editor, Mediafile) {
         return {
             // ngDialog for user form
-            getDialog: function (user) {
+            getDialog: function () {
                 return {
                     template: 'static/templates/users/profile-password-form.html',
                     controller: 'UserProfileCtrl',
@@ -411,6 +464,13 @@ angular.module('OpenSlidesApp.users.site', [
                         required: true
                     },
                 },
+                {
+                    key: 'email',
+                    type: 'input',
+                    templateOptions: {
+                        label: gettextCatalog.getString('Email')
+                    },
+                },
                 {
                     key: 'about_me',
                     type: 'editor',
@@ -432,7 +492,7 @@ angular.module('OpenSlidesApp.users.site', [
     function (gettextCatalog) {
         return {
             // ngDialog for user form
-            getDialog: function (user) {
+            getDialog: function () {
                 return {
                     template: 'static/templates/users/profile-password-form.html',
                     controller: 'UserPasswordCtrl',
@@ -476,7 +536,6 @@ angular.module('OpenSlidesApp.users.site', [
 
 .controller('UserListCtrl', [
     '$scope',
-    '$state',
     '$http',
     'ngDialog',
     'UserForm',
@@ -485,20 +544,29 @@ angular.module('OpenSlidesApp.users.site', [
     'PasswordGenerator',
     'Projector',
     'ProjectionDefault',
-    'UserListContentProvider',
     'Config',
-    'UserAccessDataListContentProvider',
-    'PdfMakeDocumentProvider',
-    'gettextCatalog',
     'UserCsvExport',
     'osTableFilter',
     'osTableSort',
+    'osTablePagination',
     'gettext',
-    'PdfCreate',
-    function($scope, $state, $http, ngDialog, UserForm, User, Group, PasswordGenerator, Projector, ProjectionDefault,
-        UserListContentProvider, Config, UserAccessDataListContentProvider, PdfMakeDocumentProvider, gettextCatalog,
-        UserCsvExport, osTableFilter, osTableSort, gettext, PdfCreate) {
-        User.bindAll({}, $scope, 'users');
+    'UserPdfExport',
+    'InvitationEmails',
+    'ErrorMessage',
+    function($scope, $http, ngDialog, UserForm, User, Group, PasswordGenerator,
+        Projector, ProjectionDefault, Config, UserCsvExport, osTableFilter, osTableSort,
+        osTablePagination, gettext, UserPdfExport, InvitationEmails, ErrorMessage) {
+        $scope.$watch(function () {
+            return User.lastModified();
+        }, function () {
+            $scope.users = _.orderBy(User.getAll(), ['first_name']);
+            _.forEach($scope.users, function (user) {
+                user.has_last_email_send = !!user.last_email_send;
+            });
+            if ($scope.updateUsers) {
+                $scope.updateUsers();
+            }
+        });
         Group.bindAll({where: {id: {'>': 1}}}, $scope, 'groups');
         $scope.$watch(function () {
             return Projector.lastModified();
@@ -538,10 +606,16 @@ angular.module('OpenSlidesApp.users.site', [
                     choiceYes: gettext('Is a committee'),
                     choiceNo: gettext('Is not a committee'),
                 },
-
+                hasLastEmailSend: {
+                    value: undefined,
+                    displayName: gettext('Last email send'),
+                    choiceYes: gettext('Got an email'),
+                    choiceNo: gettext("Didn't get an email"),
+                },
             };
         }
-        $scope.filter.propertyList = ['first_name', 'last_name', 'title', 'number', 'comment', 'structure_level'];
+        $scope.filter.propertyList = ['first_name', 'last_name', 'username', 'title',
+            'number', 'comment', 'structure_level'];
         $scope.filter.propertyDict = {
             'groups_id' : function (group_id) {
                 return Group.get(group_id).name;
@@ -551,8 +625,10 @@ angular.module('OpenSlidesApp.users.site', [
             group: function (user) {return user.groups_id;},
         };
         // Sorting
-        $scope.sort = osTableSort.createInstance();
-        $scope.sort.column = $scope.config('users_sort_by');
+        $scope.sort = osTableSort.createInstance('UserTableSort');
+        if (!$scope.sort.column) {
+            $scope.sort.column = $scope.config('users_sort_by');
+        }
         $scope.sortOptions = [
             {name: 'first_name',
              display_name: gettext('Given name')},
@@ -570,15 +646,12 @@ angular.module('OpenSlidesApp.users.site', [
              display_name: gettext('Structure level')},
             {name: 'comment',
              display_name: gettext('Comment')},
+            {name: 'last_email_send',
+             display_name: gettext('Last email send')},
         ];
 
         // pagination
-        $scope.currentPage = 1;
-        $scope.itemsPerPage = 25;
-        $scope.limitBegin = 0;
-        $scope.pageChanged = function() {
-            $scope.limitBegin = ($scope.currentPage - 1) * $scope.itemsPerPage;
-        };
+        $scope.pagination = osTablePagination.createInstance('UserTablePagination');
 
         // Toggle group from user
         $scope.toggleGroup = function (user, group) {
@@ -683,21 +756,25 @@ angular.module('OpenSlidesApp.users.site', [
                 User.save(user);
             });
         };
+        // Send invitation emails
+        $scope.sendInvitationEmails = function () {
+            InvitationEmails.send($scope.usersFiltered).then(function (success) {
+                $scope.alert = success;
+                $scope.isSelectMode = false;
+                $scope.uncheckAll();
+            }, function (error) {
+                $scope.alert = ErrorMessage.forAlert(error);
+                $scope.isSelectMode = false;
+                $scope.uncheckAll();
+            });
+        };
 
         // Export as PDF
         $scope.pdfExportUserList = function () {
-            var filename = gettextCatalog.getString("List of participants")+".pdf";
-            var userListContentProvider = UserListContentProvider.createInstance($scope.usersFiltered, $scope.groups);
-            var documentProvider = PdfMakeDocumentProvider.createInstance(userListContentProvider);
-            PdfCreate.download(documentProvider.getDocument(), filename);
+            UserPdfExport.exportUserList($scope.usersFiltered);
         };
         $scope.pdfExportUserAccessDataList = function () {
-            var filename = gettextCatalog.getString("List of access data")+".pdf";
-            var userAccessDataListContentProvider = UserAccessDataListContentProvider.createInstance(
-                $scope.usersFiltered, $scope.groups, Config);
-            var documentProvider = PdfMakeDocumentProvider.createInstance(userAccessDataListContentProvider);
-            var noFooter = true;
-            PdfCreate.download(documentProvider.getDocument(noFooter), filename);
+            UserPdfExport.exportUserAccessDataList($scope.usersFiltered);
         };
         // Export as a csv file
         $scope.csvExport = function () {
@@ -706,6 +783,89 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
+.factory('InvitationEmails', [
+    '$http',
+    'User',
+    'Config',
+    'gettextCatalog',
+    function ($http, User, Config, gettextCatalog) {
+        return {
+            // Returns the request promise. If it was successfull, a nice message for
+            // an alert is generated and the alert-object is returned.
+            send: function (users) {
+                var user_ids = _
+                    .chain(users)
+                    .filter(function (user) {
+                        return user.selected;
+                    })
+                    .map(function (user) {
+                        return user.id;
+                    })
+                    .value();
+                var subject = gettextCatalog.getString(Config.get('users_email_subject').value);
+                var message = gettextCatalog.getString(Config.get('users_email_body').value);
+
+                return $http.post('/rest/users/user/mass_invite_email/', {
+                    user_ids: user_ids,
+                    subject: subject,
+                    message: message,
+                }).then(function (success) {
+                    var numEmails = success.data.count;
+                    var noEmailIds = success.data.no_email_ids;
+                    var type = 'success', msg;
+                    if (numEmails === 0) {
+                        type = 'danger';
+                        msg = gettextCatalog.getString('No emails were send.');
+                    } else if (numEmails === 1) {
+                        msg = gettextCatalog.getString('One email was send sucessfully.');
+                    } else {
+                        msg = gettextCatalog.getString('%num% emails were send sucessfully.').replace('%num%', numEmails);
+                    }
+
+                    if (noEmailIds.length) {
+                        type = 'warning';
+                        msg += ' ';
+
+                        if (noEmailIds.length === 1) {
+                            msg += gettextCatalog.getString('The user %user% has no email, so the invitation email could not be send.');
+                        } else {
+                            msg += gettextCatalog.getString('The users %user% have no email, so the invitation emails could not be send.');
+                        }
+
+                        // This one builds a username string like "user1, user2 and user3" with the full names.
+                        var lastUsername, userString = _
+                            .chain(noEmailIds)
+                            .map(function (id) {
+                                var user = User.get(id);
+                                return user ? user.get_full_name() : '';
+                            })
+                            .filter(function (username) {
+                                return username;
+                            })
+                            .tap(function (names) {
+                                if (names.length !== 1) {
+                                    lastUsername = names.pop();
+                                }
+                            })
+                            .join(', ')
+                            .thru(function (names) {
+                                return lastUsername ? names + ' ' + gettextCatalog.getString('and') + ' ' + lastUsername : names;
+                            })
+                            .value();
+                        msg = msg.replace('%user%', userString);
+                    }
+
+                    return {
+                        msg: msg,
+                        type: type,
+                        show: true,
+                    };
+                });
+            },
+        };
+    }
+])
+
 .controller('UserDetailCtrl', [
     '$scope',
     'ngDialog',
@@ -715,9 +875,17 @@ angular.module('OpenSlidesApp.users.site', [
     'Group',
     'Projector',
     'ProjectionDefault',
-    function($scope, ngDialog, UserForm, User, userId, Group, Projector, ProjectionDefault) {
-        User.bindOne(userId, $scope, 'user');
+    'gettextCatalog',
+    'WebpageTitle',
+    function($scope, ngDialog, UserForm, User, userId, Group, Projector, ProjectionDefault, gettextCatalog,
+        WebpageTitle) {
         Group.bindAll({where: {id: {'>': 1}}}, $scope, 'groups');
+        $scope.$watch(function () {
+            return User.lastModified(userId);
+        }, function () {
+            $scope.user = User.get(userId);
+            WebpageTitle.updateTitle(gettextCatalog.getString('Participant') + ' ' + $scope.user.get_short_name());
+        });
         $scope.$watch(function () {
             return Projector.lastModified();
         }, function () {
@@ -739,10 +907,8 @@ angular.module('OpenSlidesApp.users.site', [
     '$state',
     'User',
     'UserForm',
-    'Group',
     'ErrorMessage',
-    function($scope, $state, User, UserForm, Group, ErrorMessage) {
-        Group.bindAll({where: {id: {'>': 2}}}, $scope, 'groups');
+    function($scope, $state, User, UserForm, ErrorMessage) {
         $scope.alert = {};
         // get all form fields
         $scope.formFields = UserForm.getFormFields(true);
@@ -767,14 +933,11 @@ angular.module('OpenSlidesApp.users.site', [
 .controller('UserUpdateCtrl', [
     '$scope',
     '$state',
-    '$http',
     'User',
     'UserForm',
-    'Group',
     'userId',
     'ErrorMessage',
-    function($scope, $state, $http, User, UserForm, Group, userId, ErrorMessage) {
-        Group.bindAll({where: {id: {'>': 2}}}, $scope, 'groups');
+    function($scope, $state, User, UserForm, userId, ErrorMessage) {
         $scope.alert = {};
         // set initial values for form model by create deep copy of user object
         // so list/detail view is not updated while editing
@@ -869,6 +1032,18 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
+.directive("showPassword", function() {
+    return function linkFn(scope, elem, attrs) {
+        scope.$watch(attrs.showPassword, function(value) {
+            if (value) {
+                elem.attr("type", "text");
+            } else {
+                elem.attr("type", "password");
+            }
+        });
+    };
+})
+
 .controller('UserPasswordCtrl', [
     '$scope',
     '$state',
@@ -908,46 +1083,95 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
+.controller('UserPresenceCtrl', [
+    '$scope',
+    'User',
+    'gettextCatalog',
+    'ErrorMessage',
+    function ($scope, User, gettextCatalog, ErrorMessage) {
+        $scope.alert = {};
+        $('#userNumber').focus();
+
+        $scope.changeState = function () {
+            if (!$scope.number) {
+                return;
+            }
+            var enteredNumber = $scope.number.trim();
+            var user = _.find(User.getAll(), function (user) {
+                return user.number === enteredNumber;
+            });
+            if (user) {
+                user.is_present = !user.is_present;
+                User.save(user).then(function (success) {
+                    var messageText = user.full_name + ' ' + gettextCatalog.getString('is now') + ' ';
+                    messageText += gettextCatalog.getString(user.is_present ? 'present' : 'not present') + '.';
+                    $scope.alert = {
+                        msg: messageText,
+                        show: true,
+                        type: 'success',
+                    };
+                    $scope.number = '';
+                }, function (error) {
+                    $scope.alert = ErrorMessage.forAlert(error);
+                });
+            } else {
+                $scope.alert = {
+                    msg: gettextCatalog.getString('Cannot find the participant with the participant number') + ' "' + enteredNumber + '".',
+                    show: true,
+                    type: 'danger',
+                };
+            }
+            $('#userNumber').focus();
+        };
+    }
+])
+
 .controller('UserImportCtrl', [
     '$scope',
+    '$http',
     '$q',
     'gettext',
     'gettextCatalog',
     'User',
     'Group',
     'UserCsvExport',
-    function($scope, $q, gettext, gettextCatalog, User, Group, UserCsvExport) {
+    'osTablePagination',
+    'ErrorMessage',
+    function($scope, $http, $q, gettext, gettextCatalog, User, Group, UserCsvExport,
+        osTablePagination, ErrorMessage) {
         // import from textarea
         $scope.importByLine = function () {
-            $scope.usernames = $scope.userlist[0].split("\n");
-            $scope.importcounter = 0;
-            $scope.usernames.forEach(function(name) {
+            var usernames = $scope.userlist[0].split("\n");
+            var users = _.map(usernames, function (name) {
                 // Split each full name in first and last name.
                 // The last word is set as last name, rest is the first name(s).
                 // (e.g.: "Max Martin Mustermann" -> last_name = "Mustermann")
                 var names = name.split(" ");
                 var last_name = names.slice(-1)[0];
                 var first_name = names.slice(0, -1).join(" ");
-                var user = {
+                return {
                     first_name: first_name,
                     last_name: last_name,
-                    groups_id: []
+                    groups_id: [],
                 };
-                User.create(user).then(
-                    function(success) {
-                        $scope.importcounter++;
-                    }
-                );
+            });
+            $http.post('/rest/users/user/mass_import/', {
+                users: users
+            }).then(function (success) {
+                $scope.alert = {
+                    show: true,
+                    type: 'success',
+                    msg: success.data.detail,
+                };
+            }, function (error) {
+                $scope.alert = ErrorMessage.forAlert(error);
             });
         };
 
         // pagination
-        $scope.currentPage = 1;
-        $scope.itemsPerPage = 100;
-        $scope.limitBegin = 0;
-        $scope.pageChanged = function() {
-            $scope.limitBegin = ($scope.currentPage - 1) * $scope.itemsPerPage;
-        };
+        $scope.pagination = osTablePagination.createInstance('UserImportTablePagination', 100);
+
+        // Duplicates
         $scope.duplicateActions = [
             gettext('keep original'),
             gettext('override new'),
@@ -964,9 +1188,10 @@ angular.module('OpenSlidesApp.users.site', [
         };
 
         var FIELDS = ['title', 'first_name', 'last_name', 'structure_level', 'number',
-        'groups', 'comment', 'is_active', 'is_present', 'is_committee', 'default_password'];
+        'groups', 'comment', 'is_active', 'is_present', 'is_committee', 'default_password', 'email'];
         $scope.users = [];
         $scope.onCsvChange = function (csv) {
+            $scope.csvImporting = false;
             // All user objects are already loaded via the resolve statement from ui-router.
             var users = User.getAll();
             $scope.users = [];
@@ -979,7 +1204,8 @@ angular.module('OpenSlidesApp.users.site', [
                 }
             });
             $scope.duplicates = 0;
-            _.forEach(csvUsers, function (user) {
+            _.forEach(csvUsers, function (user, index) {
+                user.importTrackId = index;
                 user.selected = true;
                 if (!user.first_name && !user.last_name) {
                     user.importerror = true;
@@ -1013,6 +1239,8 @@ angular.module('OpenSlidesApp.users.site', [
 
                     // for template:
                     user.groupsNotToCreate = _.difference(user.groups, user.groupsToCreate);
+                } else {
+                    user.groups = [];
                 }
                 user.is_active = (user.is_active !== undefined && user.is_active === '1');
                 user.is_present = (user.is_present !== undefined && user.is_present === '1');
@@ -1021,9 +1249,17 @@ angular.module('OpenSlidesApp.users.site', [
                 // Check for duplicates
                 user.duplicate = false;
                 users.forEach(function(user_) {
-                    if (user_.first_name == user.first_name &&
-                        user_.last_name == user.last_name &&
-                        user_.structure_level == user.structure_level) {
+                    user_.fullname = [
+                        user_.title,
+                        user_.first_name,
+                        user_.last_name,
+                        user_.structure_level].join(' ').trim();
+                    user.fullname = [
+                        user.title,
+                        user.first_name,
+                        user.last_name,
+                        user.structure_level].join(' ').trim();
+                    if (user_.fullname === user.fullname) {
                         if (user.duplicate) {
                             // there are multiple duplicates!
                             user.duplicate_info += '\n' + gettextCatalog.getString('There are more than one duplicates of this user!');
@@ -1112,6 +1348,11 @@ angular.module('OpenSlidesApp.users.site', [
                 var allGroups = Group.getAll();
                 var existingUsers = User.getAll();
 
+                // For option 'delete existing user' on duplicates
+                var deletePromises = [];
+                // Array of users for mass import
+                var usersToBeImported = [];
+
                 _.forEach($scope.users, function (user) {
                     if (user.selected && !user.importerror) {
                         // Assign all groups
@@ -1128,38 +1369,60 @@ angular.module('OpenSlidesApp.users.site', [
                         // Do nothing on duplicateAction==duplicateActions[0] (keep original)
                         if (user.duplicate && (user.duplicateAction == $scope.duplicateActions[1])) {
                             // delete existing user
-                            var deletePromises = [];
                             existingUsers.forEach(function(user_) {
-                                if (user_.first_name == user.first_name &&
-                                    user_.last_name == user.last_name &&
-                                    user_.structure_level == user.structure_level) {
+                                user_.fullname = [
+                                    user_.title,
+                                    user_.first_name,
+                                    user_.last_name,
+                                    user_.structure_level].join(' ').trim();
+                                user.fullname = [
+                                    user.title,
+                                    user.first_name,
+                                    user.last_name,
+                                    user.structure_level].join(' ').trim();
+                                if (user_.fullname === user.fullname) {
                                     deletePromises.push(User.destroy(user_.id));
                                 }
                             });
-                            $q.all(deletePromises).then(function() {
-                                User.create(user).then(
-                                    function(success) {
-                                        user.imported = true;
-                                    }
-                                );
-                            });
+                            usersToBeImported.push(user);
                         } else if (!user.duplicate ||
                                    (user.duplicateAction == $scope.duplicateActions[2])) {
                             // create user
-                            User.create(user).then(
-                                function(success) {
-                                    user.imported = true;
-                                }
-                            );
+                            usersToBeImported.push(user);
                         }
                     }
                 });
-                $scope.csvimported = true;
+                $q.all(deletePromises).then(function () {
+                    $http.post('/rest/users/user/mass_import/', {
+                        users: usersToBeImported
+                    }).then(function (success) {
+                        _.forEach(success.data.importedTrackIds, function (trackId) {
+                            _.find($scope.users, function (user) {
+                                return user.importTrackId === trackId;
+                            }).imported = true;
+                        });
+                        $scope.csvimported = true;
+                    }, function (error) {
+                        $scope.alert = ErrorMessage.forAlert(error);
+                    });
+                });
             });
         };
         $scope.clear = function () {
             $scope.users = null;
         };
+        $scope.excludeImportedUsers = function () {
+            $scope.users = _.filter($scope.users, function (user) {
+                return !user.imported;
+            });
+            $scope.csvImporting = false;
+            $scope.calcStats();
+        };
+        $scope.someImportedUsers = function () {
+            return _.some($scope.users, function (user) {
+                return user.imported;
+            });
+        };
         // download CSV example file
         $scope.downloadCSVExample = function () {
             UserCsvExport.downloadExample();
@@ -1346,14 +1609,15 @@ angular.module('OpenSlidesApp.users.site', [
     '$scope',
     'Group',
     'group',
+    'gettextCatalog',
     'ErrorMessage',
-    function($scope, Group, group, ErrorMessage) {
+    function($scope, Group, group, gettextCatalog, ErrorMessage) {
         $scope.group = group;
-        $scope.new_name = group.name;
+        $scope.new_name = gettextCatalog.getString(group.name);
 
         $scope.alert = {};
         $scope.save = function() {
-            var old_name = $scope.group.name;
+            var old_name = gettextCatalog.getString($scope.group.name);
             $scope.group.name = $scope.new_name;
             Group.save($scope.group).then(
                 function (success) {
@@ -1394,31 +1658,41 @@ angular.module('OpenSlidesApp.users.site', [
     }
 ])
 
-.controller('userMenu', [
-    '$scope',
+.factory('UserMenu', [
     '$http',
-    'DS',
-    'User',
-    'operator',
+    'OpenSlides',
     'ngDialog',
     'UserProfileForm',
     'UserPasswordForm',
-    function($scope, $http, DS, User, operator, ngDialog, UserProfileForm, UserPasswordForm) {
-        $scope.logout = function () {
-            $http.post('/users/logout/').then(function (response) {
-                operator.setUser(null);
-                window.location.reload();
-            });
-        };
-        $scope.editProfile = function () {
-            ngDialog.open(UserProfileForm.getDialog());
-        };
-        $scope.changePassword = function () {
-            ngDialog.open(UserPasswordForm.getDialog());
+    function ($http, OpenSlides, ngDialog, UserProfileForm, UserPasswordForm) {
+        return {
+            logout: function () {
+                $http.post('/users/logout/').then(function (response) {
+                    // Success: User logged out, so reboot OpenSlides.
+                    OpenSlides.reboot();
+                });
+            },
+            editProfile: function () {
+                ngDialog.open(UserProfileForm.getDialog());
+            },
+            changePassword: function () {
+                ngDialog.open(UserPasswordForm.getDialog());
+            },
         };
     }
 ])
 
+.controller('userMenu', [
+    '$scope',
+    'UserMenu',
+    function($scope, UserMenu) {
+        $scope.logout = UserMenu.logout;
+        $scope.editProfile = UserMenu.editProfile;
+        $scope.changePassword = UserMenu.changePassword;
+
+    }
+])
+
 .controller('LoginFormCtrl', [
     '$rootScope',
     '$scope',
@@ -1493,7 +1767,7 @@ angular.module('OpenSlidesApp.users.site', [
                 },
                 function (error) {
                     // Error: Username or password is not correct.
-                    $state.transitionTo($state.current, {msg: error.data.detail}, { 
+                    $state.transitionTo($state.current, {msg: error.data.detail}, {
                           reload: true, inherit: false, notify: true
                     });
                 }
@@ -1515,6 +1789,7 @@ angular.module('OpenSlidesApp.users.site', [
         // agenda
         gettext('Can see agenda');
         gettext('Can manage agenda');
+        gettext('Can manage list of speakers');
         gettext('Can see hidden items and time scheduling of agenda');
         gettext('Can put oneself on the list of speakers');
         // assignments
@@ -1530,6 +1805,7 @@ angular.module('OpenSlidesApp.users.site', [
         gettext('Can manage configuration');
         gettext('Can use the chat');
         gettext('Can manage the chat');
+        gettext('Can manage logos and fonts');
         // mediafiles
         gettext('Can see the list of files');
         gettext('Can upload files');
@@ -1540,7 +1816,8 @@ angular.module('OpenSlidesApp.users.site', [
         gettext('Can create motions');
         gettext('Can support motions');
         gettext('Can manage motions');
-        gettext('Can see and manage comments');
+        gettext('Can see comments');
+        gettext('Can manage comments');
         // users
         gettext('Can see names of users');
         gettext('Can see extra data of users (e.g. present and comment)');
@@ -1549,6 +1826,7 @@ angular.module('OpenSlidesApp.users.site', [
         // config strings in users/config_variables.py
         gettext('General');
         gettext('Sort name of participants by');
+        gettext('Enable participant presence view');
         gettext('Participants');
         gettext('Given name');
         gettext('Surname');
@@ -1568,6 +1846,14 @@ angular.module('OpenSlidesApp.users.site', [
         gettext('WEP');
         gettext('WPA/WPA2');
         gettext('No encryption');
+        gettext('Email');
+        gettext('Email sender');
+        gettext('Email subject');
+        gettext('Your login for {event_name}');
+        gettext('You can use {event_name} as a placeholder.');
+        gettext('Email body');
+        gettext('Dear {name},\n\nthis is your OpenSlides login for the event {event_name}:\n\n    {url}\n    username: {username}\n    password: {password}\n\nThis email was generated automatically.');
+        gettext('Use these placeholders: {name}, {event_name}, {url}, {username}, {password}. The url referrs to the system url.');
     }
 ]);
 
